@@ -1,6 +1,6 @@
 //! This crate `cpp` only provides a single macro, the `cpp!` macro. This macro
-//! by itself is not useful, but when combined with the `cpp_build` and
-//! `cpp_macro` crates it allows embedding arbitrary C++ code.
+//! by itself is not useful, but when combined with the `cpp_build` crate it
+//! allows embedding arbitrary C++ code.
 //!
 //! There are two variants of the `cpp!` macro. The first variant is used for
 //! raw text inclusion. Text is included into the generated `C++` file in the
@@ -31,9 +31,8 @@
 //!
 //! # Usage
 //!
-//! This crate must be used in tandem with the `cpp_build` and `cpp_macro`
-//! crates. A basic Cargo project which uses these projects would have a
-//! structure like the following:
+//! This crate must be used in tandem with the `cpp_build` crate. A basic Cargo
+//! project which uses these projects would have a structure like the following:
 //!
 //! ```text
 //! crate
@@ -52,11 +51,10 @@
 //! build = "build.rs"
 //!
 //! [dependencies]
-//! cpp = "0.2"
-//! cpp_macros = "0.2"
+//! cpp = "0.3"
 //!
 //! [build-dependencies]
-//! cpp_build = "0.2"
+//! cpp_build = "0.3"
 //! ```
 //!
 //! #### build.rs
@@ -74,8 +72,6 @@
 //! ```ignore
 //! #[macro_use]
 //! extern crate cpp;
-//! #[macro_use]
-//! extern crate cpp_macros;
 //!
 //! cpp!{{
 //!     #include <stdio.h>
@@ -90,6 +86,12 @@
 //! }
 //! ```
 
+#[macro_use]
+#[allow(unused_imports)]
+extern crate cpp_macros;
+#[doc(hidden)]
+pub use cpp_macros::*;
+
 /// This macro is used to embed arbitrary C++ code. See the module level
 /// documentation for more details.
 #[macro_export]
@@ -98,46 +100,12 @@ macro_rules! cpp {
 
     ([$($captures:tt)*] $($rest:tt)*) => {
         {
-            #[allow(non_camel_case_types, dead_code)]
+            #[allow(unused)]
             #[derive(__cpp_internal_closure)]
-            struct __cpp_closure(cpp! {
-                @TYPE [$($captures)*] $($rest)*
-            });
-            cpp!{@CAPTURES __cpp_closure [] => $($captures)*}
+            enum CppClosureInput {
+                Input = (stringify!([$($captures)*] $($rest)*), 0).1
+            }
+            __cpp_closure_impl![$($captures)*]
         }
     };
-
-    {@CAPTURES $name:ident
-     [$($e:expr),*] =>
-    } => {
-        $name::run($($e),*)
-    };
-
-    {@CAPTURES $name:ident
-     [$($e:expr),*] =>
-     mut $i:ident as $cty:expr , $($rest:tt)*
-    } => {
-        cpp!{@CAPTURES $name [$($e ,)* &mut $i] => $($rest)*}
-    };
-    {@CAPTURES $name:ident
-     [$($e:expr),*] =>
-     mut $i:ident as $cty:expr
-    } => {
-        cpp!{@CAPTURES $name [$($e ,)* &mut $i] =>}
-    };
-
-    {@CAPTURES $name:ident
-     [$($e:expr),*] =>
-     $i:ident as $cty:expr , $($rest:tt)*
-    } => {
-        cpp!{@CAPTURES $name [$($e ,)* &$i] => $($rest)*}
-    };
-    {@CAPTURES $name:ident
-     [$($e:expr),*] =>
-     $i:ident as $cty:expr
-    } => {
-        cpp!{@CAPTURES $name [$($e ,)* &$i] =>}
-    };
-
-    (@TYPE $($rest:tt)*) => { () };
 }
